package de.fraunhofer.iem.dataprovider.repository.service

import de.fraunhofer.iem.dataprovider.gitlab.service.OpenCodeGitlabApi
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryCreateDto
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.repository.repository.RepositoryRepository
import org.springframework.stereotype.Service
import java.util.*

@Service
class RepositoryService(
    private val repositoryRepository: RepositoryRepository,
    private val openCodeGitlabApi: OpenCodeGitlabApi
) {
    private val logger = getLogger(javaClass)

    /**
     * Either creates or returns a repository entity based upon its
     * opencode repository id.
     */
    fun getOrCreate(gitRepository: RepositoryCreateDto): RepositoryEntity {
        logger.info("Checking if repository with ${gitRepository.id} exists")
        var repo = repositoryRepository.findByRepoId(gitRepository.id)
        if (repo == null) {
            logger.info("Creating repository with id ${gitRepository.id}")
            repo = repositoryRepository.save(gitRepository.toDbObject())
        }
        return repo
    }

    /**
     * Queries the gitlab api to get the repo name, url, and id.
     */
    fun getRepositoryInfo(repoId: Long): RepositoryCreateDto {
        return openCodeGitlabApi.getRepositoryInfo(repoId)
    }

    fun findRepoByID(id: UUID): RepositoryEntity? {
        val repository = repositoryRepository.findById(id)
        if (repository.isEmpty) {
            return null
        }
        return repository.get()
    }

    fun findRepoById(id: Long): RepositoryEntity? {
        return repositoryRepository.findByRepoId(id)
    }

    fun getAllRepositories(): List<RepositoryEntity> {
        return repositoryRepository.findAll()
    }
}
